#!/usr/bin/perl -w
# redirect.pl
# generiert basierend auf der globalen Config
# und der .redirect pro user die virtusertable
# und aliases und ruft anschliessend make in
# /etc/mail auf.

use strict;
use Config::General;
use Data::Dumper;

sub d;
sub var;

use vars qw($global %verteiler $debug $conf_obj %global $aliases_static $virt_static %redirect);

$global = shift;
if (!$global) {
  print STDERR "Usage: $0 configfile\n";
  exit 1;
}

# set debugging on commandline!
$debug = shift;

$conf_obj = new Config::General($global);
%global = $conf_obj->getall;


#print Dumper(\%global);


# first load static files
open ALIASES_STATIC, "<$global{system}->{maildir}/aliases.static" or
           die "Could not read $global{system}->{maildir}/aliases.static: $!\n";
$aliases_static = join "", <ALIASES_STATIC>;
close ALIASES_STATIC;

open VIRT_STATIC, "<$global{system}->{maildir}/virtusertable.static" or
          die "Could not read $global{system}->{maildir}/virtusertable.static: $!\n";
$virt_static = join "", <VIRT_STATIC>;
close VIRT_STATIC;



# now open destination files
open ALIASES, ">$global{system}->{maildir}/aliases" or
          die "Could not open $global{system}->{maildir}/aliases for writing: $!\n";


open VIRT, ">$global{system}->{maildir}/virtusertable" or
          die "Could no open $global{system}->{maildir}/virtusertable for writing: $!\n";

# save static definitions
print ALIASES $aliases_static;
print VIRT    $virt_static;


print ALIASES qq(
#######################################################
###  Automatisch durch $0 verwalteter Teil.
###  Bitte nicht editieren, nimm statt dessen:
###  $global{system}->{maildir}/aliases.static
#######################################################
);

print VIRT qq(
#######################################################
###  Automatisch durch $0 verwalteter Teil.
###  Bitte nicht editieren, nimm statt dessen:
###  $global{system}->{maildir}/virtusertable.static
#######################################################
);



# now iterate over all domains
foreach my $domain (keys %{$global{domains}}) {
  d("processing $domain");
  my $user = $global{domains}->{$domain};
  my $redirect = "$global{system}->{homedir}/$user/.redirect";
  if (-e "$redirect") {
    d("reading $redirect");
    my($re_obj,%redirect);
    eval {
      $re_obj = new Config::General($redirect);
      %redirect = $re_obj->getall;
    };
    if ($@) {
      d("$redirect contains invalid syntax - using default. \@$domain => $user");
      d("Error was: $@! using default \@$domain\t\t$user");
      print VIRT "\@$domain\t\t$user\n";
    }
    elsif(grep {$_ =~ /(\||`)/} @{$re_obj->{content}})  {
	d("$redirect contains forbidden characters! using default \@$domain\t\t$user");
	print VIRT "\@$domain\t\t$user\n";
    }
    else {
      #print Dumper(\%redirect);
      d("$redirect loaded");
      if (exists $redirect{domain}->{$domain}) {
	d("$redirect defines $domain - fine.");
	# iterate over all addresses, longest first
	foreach my $address (sort {length($b) <=> length($a)} keys %{$redirect{domain}->{$domain}}) {
	  my $dest = $redirect{domain}->{$domain}->{$address};
	  d("process $address => $dest");
	  # translate variables, cool huh?
	  $dest    = var($dest);
	  $address = var($address);
	  d("regexped $address => $dest");
	  # now look, what it is
	  if ($dest =~ /^%(.+?)$/ ) {
	    d("$address matches to a list");
	    my $list = $1;
	    my $listname = $user . "_list_" . $list; # should be uniqe
	    if (exists $redirect{list}->{$list}->{rcpt}) {
	      d("$list is defined");
	      my $rcpts = $redirect{list}->{$list}->{rcpt};
	      my @rcpts;
	      if (ref($rcpts) eq "ARRAY") {
		@rcpts = @{$rcpts};
	      }
	      else {
		@rcpts = ($rcpts);
	      }
	      d("store list in \%verteiler");
	      $verteiler{$listname} = join ", ", map { var($_) } @rcpts;
	      d("write virtuser entry $address\@$domain\t\t$listname");
	      print VIRT "$address\@$domain\t\t$listname\n";
	    }
	  }
	  elsif ($address eq "\@") {
	    d("default clause found \@$domain\t\t$dest");
	    print VIRT "\@$domain\t\t$dest\n";
	  }
	  else {
	    d("standard entry $address\@$domain => $dest");
	    print VIRT "$address\@$domain\t\t$dest\n";
	  }
	}
      }
    }
  }
  else {
    d("$redirect does not exist");
    d("using default: \@$domain => $global{domains}->{$domain}");
    print VIRT "\@$domain\t\t$global{domains}->{$domain}\n";
  }
}



if (%verteiler) {
  d("save lists");
  foreach my $listname (sort keys %verteiler) {
    d("saving $listname");
    print ALIASES "$listname: $verteiler{$listname}\n\n";
  }
}

print "done\n";

exit 0;

sub d {
  my(@lines) = @_;
  if ($debug) {
    print "@lines\n" if($debug);
  }
  else {
    print ".";
  }
}



sub var {
  my $string = shift;
  $string =~ s/\$(.+)/( exists $redirect{var}->{$1} ? $redirect{var}->{$1} : "" )/ge;
  $string;
}
